@@ -6,12 +6,13 @@
 
 #
 # Arguments
-#'@param  y  vector containing the dependent variable. 
-#'@param  X  matrix containing the regressors, with the endogenous variables occupying the last columns.
-#'@param  P  matrix containing the discrete endogeneous regressors.
-#'@param  intercept  the intercept of the model. It should be specified whether the model should be estimated with or without intercept. 
-#'If no intercept is desired, intercept should be given the value "FALSE", otherwise the value "TRUE".
-#'@param data  optional matrix or data frame containing the dataset used.
+#'@param  formula  the model formula, e.g. \code{y ~ X1 + X2 + P}.
+#'@param  endoVar  a string with the name of the endogenous variable/s, in quotation marks.
+#'@param  intercept  an optional parameter. The model is estimated by default with 
+#'intercept. If no intercept is desired or the regressors matrix \code{X} contains already
+#'a column of ones, intercept should be given the value "FALSE", otherwise the value "TRUE".
+#'@param data a data frame or matrix containing the variables of the model.
+
 
 #
 # Return Value
@@ -20,14 +21,23 @@
 
 #'@keywords internal
 
-copulaDiscrete <- function(y,X,P, intercept = NULL, data=NULL) {
+copulaDiscrete <- function(formula, endoVar, intercept = NULL, data) {
 
+  mf <- model.frame(formula = formula, data = data)
+  y <- mf[,1]
+  regs <- unlist(strsplit(as.character(formula)[3], "\\(")) 
+  predictors <- unlist(strsplit(regs[1], " [+] "))
 
+  regressors <- mf[,2:dim(mf)[2]]
+  X <- regressors # both endogenous and exogenous
+  P <- as.matrix(X[,which(colnames(X) %in% endoVar)])
+  colnames(P) <- endoVar
+ 
  sim <- 250  
  k1 <- ncol(X)   
 # P is the endogeneous regressor
- P1 <- as.matrix(P)
- colnames(P1) <- colnames(X)[-c(1:(ncol(X)-ncol(P1)))]
+# P1 <- as.matrix(P)
+# colnames(P1) <- colnames(X)[-c(1:(ncol(X)-ncol(P1)))]
 
 #  empirical cumulative distribution function
 	 getEcdf <- function(x){
@@ -47,20 +57,20 @@ copulaDiscrete <- function(y,X,P, intercept = NULL, data=NULL) {
 
 # each element of pecdf had 3 levels - H.p, H.p0 and H.p1. 
 # Nr. of elements of pecdf = Nr. columns P1
- pecdf <- apply(P1,2,getEcdf)
+ pecdf <- apply(P,2,getEcdf)
 
 
-  U.p <- matrix(NA, dim(P1)[1], dim(P1)[2])
-  nc <- ncol(X) + ncol(P1)
+  U.p <- matrix(NA, dim(P)[1], dim(P)[2])
+  ifelse (intercept==TRUE, nc <- ncol(X) + ncol(P) +1, nc <- ncol(X) + ncol(P))
   # create a list with nc elements (number of regressors + Pstar) , for each regressor save the confint for each lm out of sim simulations
   conf.int <- rep(list(matrix(0,sim,2)), nc)
 
-  for (k in 1:dim(P1)[2]){
+  for (k in 1:dim(P)[2]){
   
-    U.p[,k] <- stats::runif(dim(P1)[1],min=pecdf[[k]]$H.p0,max=pecdf[[k]]$H.p1)
+    U.p[,k] <- stats::runif(dim(P)[1],min=pecdf[[k]]$H.p0,max=pecdf[[k]]$H.p1)
   }  
   p.star <- apply(U.p, 2,qnorm)
-  colnames(p.star) <- paste("PStar",1:ncol(P1),sep=".")
+  colnames(p.star) <- paste("PStar",1:ncol(P),sep=".")
   
   X1 <- cbind(X,p.star)
   X1 <- as.matrix(X1)
@@ -80,25 +90,25 @@ copulaDiscrete <- function(y,X,P, intercept = NULL, data=NULL) {
   
   
   for (i in 1:sim){
-  for (k in 1:dim(P1)[2]){
+  for (k in 1:dim(P)[2]){
     
-    U.p[,k] <- stats::runif(dim(P1)[1],min=pecdf[[k]]$H.p0,max=pecdf[[k]]$H.p1)
+    U.p[,k] <- stats::runif(dim(P)[1],min=pecdf[[k]]$H.p0,max=pecdf[[k]]$H.p1)
   }  
   p.star <- apply(U.p, 2,qnorm)
-  colnames(p.star) <- paste("PStar",1:ncol(P1),sep=".")
+  colnames(p.star) <- paste("PStar",1:ncol(P),sep=".")
      
   X1 <- cbind(X,p.star)
   X1 <- as.matrix(X1)
   k2 <- ncol(X1)
   dataCopula <- X1
 
-  if (intercept==FALSE){  # no intercept 
-  meth.2 <- stats::lm(y ~.-1,data=data.frame(dataCopula))
+  if (intercept==TRUE){  # intercept 
+  meth.2 <- stats::lm(y ~.,data=data.frame(dataCopula))
   for (s in 1:nc) {
            conf.int[[s]][i,] <- confint(meth.2, level=0.95)[s,]
            }
     } else {
-    meth.2 <- stats::lm(y ~.,data=data.frame(dataCopula))  # with intercept
+    meth.2 <- stats::lm(y ~.-1,data=data.frame(dataCopula))  # with intercept
    for (s in 1:nc) {
       conf.int[[s]][i,] <- confint(meth.2, level=0.95)[s,]
       
@@ -112,6 +122,6 @@ for (i in 1:nc)
   CI[i,] <- apply(conf.int[[i]],2,mean)
 
 
-res <- list(coefficients= bhat, CI=CI, reg=dataCopula, resid = meth.2$residuals, fitted.values=meth.2$fitted.values)
+res <- list(coefficients = meth.2$coefficients, CI=CI, reg = dataCopula, resid = meth.2$residuals, fitted.values=meth.2$fitted.values)
 return(res)
 }
\ No newline at end of file
